@@ -19,11 +19,10 @@
 // `error_chain!` can recurse deeply
 #![recursion_limit = "1024"]
 
-#[macro_use]
-extern crate error_chain;
 #[cfg(test)]
 extern crate pretty_assertions;
 
+use snafu::{ensure, Snafu};
 use std::cmp::min;
 use std::fs::{rename, File, OpenOptions};
 use std::io;
@@ -31,32 +30,28 @@ use std::io::prelude::*;
 use std::io::SeekFrom;
 use std::path::Path;
 
-use bytes::{Buf, BufMut, BytesMut, IntoBuf};
+use bytes::{Buf, BufMut, BytesMut};
 
-error_chain! {
-    foreign_links {
-        Io(::std::io::Error);
-    }
-
-    errors {
-        TooManyElements {
-            description("too many elements")
-        }
-        ElementTooBig {
-            description("element too big")
-        }
-        CorruptedFile(msg: String) {
-            description("corrupted file")
-            display("corrupted file: {}", msg)
-        }
-        UnsupportedVersion(detected: u32, supported: u32) {
-            description("unsupported version")
-            display("unsupported version {}. supported versions is {} and legacy",
-                detected, supported)
-        }
-    }
+#[derive(Debug, Snafu)]
+pub enum Error {
+    #[snafu(context(false))]
+    Io { source: std::io::Error },
+    #[snafu(display("too many elements"))]
+    TooManyElements {},
+    #[snafu(display("element too big"))]
+    ElementTooBig {},
+    #[snafu(display("corrupted file: {}", msg))]
+    CorruptedFile { msg: String },
+    #[snafu(display(
+        "unsupported version {}. supported versions is {} and legacy",
+        detected,
+        supported
+    ))]
+    UnsupportedVersion { detected: u32, supported: u32 },
 }
 
+type Result<T, E = Error> = std::result::Result<T, E>;
+
 /// QueueFile is a lightning-fast, transactional, file-based FIFO.
 ///
 /// Addition and removal from an instance is an O(1) operation and is atomic.
@@ -159,10 +154,10 @@ impl QueueFile {
             let mut buf = BytesMut::with_capacity(16);
 
             if force_legacy {
-                buf.put_u32_be(QueueFile::INITIAL_LENGTH as u32);
+                buf.put_u32(QueueFile::INITIAL_LENGTH as u32);
             } else {
-                buf.put_u32_be(QueueFile::VERSIONED_HEADER);
-                buf.put_u64_be(QueueFile::INITIAL_LENGTH);
+                buf.put_u32(QueueFile::VERSIONED_HEADER);
+                buf.put_u64(QueueFile::INITIAL_LENGTH);
             }
 
             file.write_all(buf.as_ref())?;
@@ -196,9 +191,7 @@ impl QueueFile {
         file.seek(SeekFrom::Start(0))?;
         let bytes_read = file.read(&mut buf)?;
 
-        if bytes_read < 32 {
-            bail!(ErrorKind::CorruptedFile("file too short".into()));
-        }
+        ensure!(bytes_read >= 32, CorruptedFile { msg: "file too short" });
 
         let versioned = !force_legacy && (buf[0] & 0x80) != 0;
 
@@ -208,21 +201,19 @@ impl QueueFile {
         let first_pos: u64;
         let last_pos: u64;
 
-        let mut buf = buf.into_buf();
+        let mut buf = BytesMut::from(&buf[..]);
 
         if versioned {
             header_len = 32;
 
-            let version = buf.get_u32_be() & 0x7FFF_FFFF;
+            let version = buf.get_u32() & 0x7FFF_FFFF;
 
-            if version != 1 {
-                bail!(ErrorKind::UnsupportedVersion(version, 1));
-            }
+            ensure!(version == 1, UnsupportedVersion { detected: version, supported: 1u32 });
 
-            file_len = buf.get_u64_be();
-            elem_cnt = buf.get_u32_be() as usize;
-            first_pos = buf.get_u64_be();
-            last_pos = buf.get_u64_be();
+            file_len = buf.get_u64();
+            elem_cnt = buf.get_u32() as usize;
+            first_pos = buf.get_u64();
+            last_pos = buf.get_u64();
 
             assert!(file_len <= i64::max_value() as u64);
             assert!(elem_cnt <= i32::max_value() as usize);
@@ -231,10 +222,10 @@ impl QueueFile {
         } else {
             header_len = 16;
 
-            file_len = u64::from(buf.get_u32_be());
-            elem_cnt = buf.get_u32_be() as usize;
-            first_pos = u64::from(buf.get_u32_be());
-            last_pos = u64::from(buf.get_u32_be());
+            file_len = u64::from(buf.get_u32());
+            elem_cnt = buf.get_u32() as usize;
+            first_pos = u64::from(buf.get_u32());
+            last_pos = u64::from(buf.get_u32());
 
             assert!(file_len <= i32::max_value() as u64);
             assert!(elem_cnt <= i32::max_value() as usize);
@@ -244,30 +235,21 @@ impl QueueFile {
 
         let real_file_len = file.metadata()?.len();
 
-        if file_len > real_file_len {
-            bail!(ErrorKind::CorruptedFile(format!(
+        ensure!(file_len <= real_file_len, CorruptedFile {
+            msg: format!(
                 "file is truncated. expected length was {} but actual length is {}",
                 file_len, real_file_len
-            )));
-        }
-        if file_len <= header_len {
-            bail!(ErrorKind::CorruptedFile(format!(
-                "length stored in header ({}) is invalid",
-                file_len
-            )));
-        }
-        if first_pos > file_len {
-            bail!(ErrorKind::CorruptedFile(format!(
-                "position of the first element ({}) is beyond the file",
-                first_pos
-            )));
-        }
-        if last_pos > file_len {
-            bail!(ErrorKind::CorruptedFile(format!(
-                "position of the last element ({}) is beyond the file",
-                last_pos
-            )));
-        }
+            )
+        });
+        ensure!(file_len >= header_len, CorruptedFile {
+            msg: format!("length stored in header ({}) is invalid", file_len)
+        });
+        ensure!(first_pos <= file_len, CorruptedFile {
+            msg: format!("position of the first element ({}) is beyond the file", first_pos)
+        });
+        ensure!(last_pos <= file_len, CorruptedFile {
+            msg: format!("position of the last element ({}) is beyond the file", last_pos)
+        });
 
         let mut queue_file = QueueFile {
             file,
@@ -321,15 +303,11 @@ impl QueueFile {
 
     /// Adds an element to the end of the queue.
     pub fn add(&mut self, buf: &[u8]) -> Result<()> {
-        if self.elem_cnt + 1 > i32::max_value() as usize {
-            bail!(ErrorKind::TooManyElements);
-        }
+        ensure!(self.elem_cnt + 1 < i32::max_value() as usize, TooManyElements {});
 
         let len = buf.len();
 
-        if len > i32::max_value() as usize {
-            bail!(ErrorKind::ElementTooBig);
-        }
+        ensure!(len <= i32::max_value() as usize, ElementTooBig {});
 
         self.expand_if_necessary(len)?;
 
@@ -493,21 +471,21 @@ impl QueueFile {
             assert!(first_pos <= i64::max_value() as u64);
             assert!(last_pos <= i64::max_value() as u64);
 
-            self.header_buf.put_u32_be(QueueFile::VERSIONED_HEADER);
-            self.header_buf.put_u64_be(file_len);
-            self.header_buf.put_i32_be(elem_cnt as i32);
-            self.header_buf.put_u64_be(first_pos);
-            self.header_buf.put_u64_be(last_pos);
+            self.header_buf.put_u32(QueueFile::VERSIONED_HEADER);
+            self.header_buf.put_u64(file_len);
+            self.header_buf.put_i32(elem_cnt as i32);
+            self.header_buf.put_u64(first_pos);
+            self.header_buf.put_u64(last_pos);
         } else {
             assert!(file_len <= i32::max_value() as u64);
             assert!(elem_cnt <= i32::max_value() as usize);
             assert!(first_pos <= i32::max_value() as u64);
             assert!(last_pos <= i32::max_value() as u64);
 
-            self.header_buf.put_i32_be(file_len as i32);
-            self.header_buf.put_i32_be(elem_cnt as i32);
-            self.header_buf.put_i32_be(first_pos as i32);
-            self.header_buf.put_i32_be(last_pos as i32);
+            self.header_buf.put_i32(file_len as i32);
+            self.header_buf.put_i32(elem_cnt as i32);
+            self.header_buf.put_i32(first_pos as i32);
+            self.header_buf.put_i32(last_pos as i32);
         }
 
         self.seek(0)?;
@@ -778,7 +756,7 @@ mod tests {
     use super::*;
 
     fn gen_rand_data(size: usize) -> Box<[u8]> {
-        let mut buf = vec![0u8; size];;
+        let mut buf = vec![0u8; size];
         thread_rng().fill(buf.as_mut_slice());
 
         buf.into_boxed_slice()
